"use strict";
this.name        = "explorer_club.js"; 
this.author      = "Capt. Murphy"; 
this.copyright   = "2011 Capt Murphy";
this.licence     = "CC BY-NC-SA 3.0"; 
this.description = "Worldscript to record unique systems visited."; 
this.version     = "1.3.3.oxpC";

// OXPConfig settings. Variables have friendlier names for display on OXPconfig screens. Converted to format used by the main script by oxpcNotifyOnChange.
this.oxpcSettings = {
	Info: {Name:"explorer_club.js",Display:"Explorers' Club",Notify:true,
		InfoB:"Record Visit.\nIf false a visit will be recorded on docking with the main station.\nIf true a visit will be recorded on arrival from witchspace.\nPhotographic record. False: Off. True: On.",
		InfoS:"Long range chart behaviour.\nSlider point: 0- Standard behaviour. 1- Systems visited to date will be marked.\n2- Systems not visited to date will be marked. 3- F7 will mark targeted system.\n4- F7 will unmark targeted system."},
	Bool0: {Name:"Record_visit_option",Def:false,Desc:"Record visit."},
	Bool1: {Name:"Snapshot_option",Def:false,Desc:"Photographic record."},
	SInt0: {Name:"Long_range_chart_option",Def:0x0,Max:0x4,Desc:"Long Range Chart."}
	}
	
// OXPConfig2 notification - this is also called manually when a save game is loaded to set the variables used by the main script.
this.oxpcNotifyOnChange = function(what)
{
 if (!this.Record_visit_option)
 {
    if (this.explorerClub_visitOption !== 1)
	{this.explorerClub_visitOption = 1;}
 }
 if (this.Record_visit_option)
 {
    if (this.explorerClub_visitOption !== 2)
	{this.explorerClub_visitOption = 2;}
 }
 if (!this.Snapshot_option)
 {
	if (this.explorerClub_snapshotOption !== 1)
	{this.explorerClub_snapshotOption = 1}
 }
 if (this.Snapshot_option)
 {
	if (this.explorerClub_snapshotOption !== 2)
	{
	  this.explorerClub_snapshotOption = 2;
	  if (this.explorerClub_visitOption === 1 && player.ship.dockedStation.isMainStation && this.explorerClub_record[galaxyNumber].lastIndexOf(system.ID) === this.explorerClub_record[galaxyNumber].length - 1)
    {this.explorerClub_snapshotOnLaunch = true;}
	}
 }
 switch (this.Long_range_chart_option)
 {
  case 0x0:
  this.explorerClub_chartOption = 1;
  break;
  case 0x1:
  this.explorerClub_chartOption = 2;
  break;
  case 0x2:
  this.explorerClub_chartOption = 5;
  break;
  case 0x3:
  this.explorerClub_chartOption = 3;
  break;
  case 0x4:
  this.explorerClub_chartOption = 4;
  }
 }

this.startUp = function()
{	
    // if no record of systems visited in save game, sets up arrays
	if (!missionVariables.explorerClub_record) 
	{
		this.explorerClub_record = new Array (8);
		var galCounter = 0;
		for (galCounter = 0; galCounter < 8;galCounter++)
		{this.explorerClub_record[galCounter] = new Array();}
		if (missionVariables.framesave !== "MoveToOxpStation") // check that first run isn't with a Save Anywhere save file.
		{this.explorerClub_record[galaxyNumber][0] = system.ID;} // and if not records the current system as visited.
		// sets default options for chart and visit behaviour.
		this.explorerClub_chartOption = 1;
		this.Long_range_chart_option = 0x0;
		this.explorerClub_visitOption = 1;
		this.Record_visit_option = false;
		this.explorerClub_snapshotOption = 1;
		this.Snapshot_option = false;
		return;
	}
	// otherwise load visit record and options from save game.
	this.explorerClub_record = JSON.parse(missionVariables.explorerClub_record);
	if (missionVariables.explorerClub_snapshotOnLaunch)
	{this.explorerClub_snapshotOnLaunch = missionVariables.explorerClub_snapshotOnLaunch;}
	if (missionVariables.explorerClub_visitOption) // convert other missionVariables to OXPconfig format if present on first run
	{this.explorerClub_convertVariables();}
	else {this.oxpcNotifyOnChange(0);} // otherwise update variables from OXPconfig settings.
	// check for galaxyNumber out of range and if so disable OXP.
	if (galaxyNumber < 0 || galaxyNumber > 7)
	{log(this.name,"Warning - Explorers' Club is not compatible with additional hacked galaxies. OXP disabled.");
	player.consoleMessage("Warning - Explorers' Club is not compatible with additional hacked galaxies. OXP disabled.",10);
	delete this.oxpcSettings;
	delete this.oxpcNotifyOnChange;
	delete this.playerWillSaveGame;
	delete this.shipWillDockWithStation;
	delete this.shipWillExitWitchspace;
	delete this.explorerClub_recordVisit;
	delete this.guiScreenWillChange;
    delete this.guiScreenChanged;
	delete this.explorerClub_markMap;
	delete this.explorerClub_systemList;
	delete this.explorerClub_mileStones;
	delete this.missionScreenOpportunity;
	delete this.explorerClub_isArray1_subsetOfArray2;
	delete this.shipExitedWitchspace;
	delete this.explorerClub_freezePlayer;
	delete this.shipLaunchedFromStation;
	delete this.explorerClub_positionPlayer;
	delete this.explorerClub_removeCallback;
	}
}
// function to convert existing missionVariables to oxpConfig2 format before deleting them.
this.explorerClub_convertVariables = function()
{
 if (missionVariables.explorerClub_visitOption === 1)
 {this.explorerClub_visitOption = 1; this.Record_visit_option = false;}
 else {this.explorerClub_visitOption = 2; this.Record_visit_option = true;}
 if (missionVariables.explorerClub_chartOption === 1)
 {this.explorerClub_chartOption = 1; this.Long_range_chart_option = 0x0;}
 if (missionVariables.explorerClub_chartOption === 2)
 {this.explorerClub_chartOption = 2; this.Long_range_chart_option = 0x1;}
 if (missionVariables.explorerClub_chartOption === 3)
 {this.explorerClub_chartOption = 3; this.Long_range_chart_option = 0x3;}
 if (missionVariables.explorerClub_chartOption === 4)
 {this.explorerClub_chartOption = 4; this.Long_range_chart_option = 0x4;}
 if (missionVariables.explorerClub_chartOption === 5)
 {this.explorerClub_chartOption = 5; this.Long_range_chart_option = 0x2;}
 delete missionVariables.explorerClub_chartOption;
 delete missionVariables.explorerClub_visitOption;
 if (!missionVariables.explorerClub_snapshotOption || missionVariables.explorerClub_snapshotOption === 1)
 {this.explorerClub_snapshotOption = 1; this.Snapshot_option = false;}
 else {this.explorerClub_snapshotOption = 2; this.Snapshot_option = true;}
 delete missionVariables.explorerClub_snapshotOption;
}
 
this.playerWillSaveGame = function ()
{
	missionVariables.explorerClub_record = JSON.stringify(this.explorerClub_record); // save record of systems visited
	if (this.explorerClub_snapshotOnLaunch) // if a snapshot on launch isn scheduled save that as a missionVariable
	{missionVariables.explorerClub_snapshotOnLaunch = this.explorerClub_snapshotOnLaunch;}
	else {delete missionVariables.explorerClub_snapshotOnLaunch;}
}

this.shipWillDockWithStation = function(station)
{
	// don't record anything if interstellar space or not a main station, or visit option is set to flying visit. Within version 1.3 added a check to see if dock event is scripted by Save Anywhere OXP and if so a visit is not recorded.
    if (system.isInterstellarSpace || !station.isMainStation || this.explorerClub_visitOption === 2 || player.ship.equipmentStatus("EQ_FRAME_SAVE") === "EQUIPMENT_OK")
	{return;}
	this.explorerClub_recordVisit();
}

this.shipExitedWitchspace = function()
{
    // don't record anything if interstellar space or visit option is set to station dock only.
    if (system.isInterstellarSpace || this.explorerClub_visitOption === 1) 
	{return;}
	this.explorerClub_recordVisit();
	this.explorerClub_arrivalSnapshotCheck();
}

this.explorerClub_recordVisit = function()
{
    // check to see if new system has already been visited and if not add it to the array
	// thanks to Svengali for pointing out JS function indexOf - I hadn't seen its use documented. Very handy.
	if(this.explorerClub_record[galaxyNumber].indexOf(system.ID)===-1)
	{
		this.explorerClub_record[galaxyNumber].push(system.ID);
		// if snapshot feature is on set the appropriate flag to trigger appropriate snapshot code.
		if (this.explorerClub_snapshotOption)
		{
		  if (this.explorerClub_visitOption === 1)
		  {this.explorerClub_snapshotOnLaunch = true;}
		  else {this.explorerClub_snapshotOnArrival = true;}
		}
	}
}

this.guiScreenWillChange = function(to, from)
{
    if (to === "GUI_SCREEN_MANIFEST")
	{
	// counts unique systems visited in current galaxy and overall for display on manifest screen.
	 var galaxyVisited = this.explorerClub_record[galaxyNumber].length;
	 var ooniverseVisited = 0;
	 var galCounter = 0;
	 for (galCounter = 0; galCounter < 8;galCounter++)
	 {ooniverseVisited += this.explorerClub_record[galCounter].length;}
     var rank;
	 if (ooniverseVisited === 2048)
	 {rank = "**Elite Explorer**.";}
	 if (ooniverseVisited < 2048 && ooniverseVisited >= 1024)
	 {rank = "Explorer Extraordinaire.";}
	 if (ooniverseVisited < 1024 && ooniverseVisited >= 512)
	 {rank = "Master Explorer.";}
	 if (ooniverseVisited < 512 && ooniverseVisited >= 256)
	 {rank = "Journeyman Explorer.";}
	 if (ooniverseVisited < 256 && ooniverseVisited >= 128)
	 {rank = "Apprentice Explorer.";}
	 if (ooniverseVisited < 128) 
	 {rank = "Novice Explorer.";}
	 var manifestString = "Systems visited: Current galaxy - " + galaxyVisited + "/256. Ooniverse - " + ooniverseVisited +"/2048. Rank - " + rank;
	 mission.setInstructions (manifestString, "explorer_club.js");
	}
}

this.guiScreenChanged = function(to,from)
{
    // code block for access to achievements screens.
    if (to === "GUI_SCREEN_MANIFEST")
	{
		player.consoleMessage("Press F6 to view your Explorers' Club achievements.",6);
		return;
	}
	if (to === "GUI_SCREEN_SHORT_RANGE_CHART" && from === "GUI_SCREEN_MANIFEST")
	{
		this.explorerClub_systemList();
		return;
	}
	//code block for entering long range chart. Behaviour depends on what options has been set.
	if (to === "GUI_SCREEN_LONG_RANGE_CHART" && guiScreen === "GUI_SCREEN_LONG_RANGE_CHART")
	{
		switch (this.explorerClub_chartOption)
		{
			case 1:
			{}
			break;
			case 2:
			{
			    var counter = 0;
				for (counter = 0; counter < 256; counter++)
				{mission.unmarkSystem(counter);}
				player.consoleMessage("All systems visited are marked in order of visit. Press F7 to clear marked systems.",10);
				this.explorerClub_counter = 0;
				// double timer to execute function every 0.125 seconds.
				this.explorerClub_markTimer = new Timer(this, this.explorerClub_markMap, 0.125, 0.25);
				this.explorerClub_markTimer1 = new Timer(this, this.explorerClub_markMap, 0.25, 0.25);
			}break;
			case 3:			
			{
				player.consoleMessage("Press F7 to mark current witch-space target.",10);
			}break;
			case 4:
			{
				player.consoleMessage("Press F7 to unmark current witch-space target.",10);
			}break;	
			case 5:
			{
			    var counter = 0;
				for (counter = 0; counter < 256; counter++)
				{mission.unmarkSystem(counter);}
				player.consoleMessage("Systems not visited so far are marked. Press F7 to clear marked systems.",10);
				var systemCounter = 0;
				var currentGalaxy = galaxyNumber;
				for (systemCounter = 0; systemCounter < 256;systemCounter++)
				{
					if(this.explorerClub_record[currentGalaxy].indexOf(systemCounter)===-1)
					{
						mission.markSystem(systemCounter);
					}
				}
			}break;
		}return;
	}
	// code block for exiting Long Range Chart.
	if (from === "GUI_SCREEN_LONG_RANGE_CHART")
	{
		if (this.explorerClub_markTimer) {this.explorerClub_markTimer.stop();delete this.explorerClub_markTimer;}
		if (this.explorerClub_markTimer1) {this.explorerClub_markTimer1.stop();delete this.explorerClub_markTimer1;}
		if (to === "GUI_SCREEN_SYSTEM_DATA")
		{
			if (this.explorerClub_chartOption === 3)
			{
				mission.markSystem(player.ship.targetSystem);
				return;
			}	
			if (this.explorerClub_chartOption === 4)
			{
				mission.unmarkSystem(player.ship.targetSystem);
				return;
			}	
			if (this.explorerClub_chartOption === 2 || this.explorerClub_chartOption === 5)
			{
				var counter = 0;
				for (counter = 0; counter < 256; counter++)
				{mission.unmarkSystem(counter);}
			}
			// added with version 1.3 - appends a short note to the system description depending on whether the system shown on the screen is recorded as visited or not.
			if (guiScreen === "GUI_SCREEN_SYSTEM_DATA")
			{
				if(this.explorerClub_record[galaxyNumber].indexOf(player.ship.targetSystem) !== -1) {mission.addMessageText("\nExplorers' Club - You have already visited this system.");} else {mission.addMessageText("\nExplorers' Club - You have not visited this system.");}
			}	
		}
	}
}
// function to animate systems visited - called from timers.
this.explorerClub_markMap = function()
{
if (this.explorerClub_counter >= this.explorerClub_record[galaxyNumber].length)
 {
  if (this.explorerClub_markTimer)
  {
	this.explorerClub_markTimer.stop();
	delete this.explorerClub_markTimer;
  }
  if (this.explorerClub_markTimer1)
  {
	this.explorerClub_markTimer1.stop();
	delete this.explorerClub_markTimer1;
  }
  return;
 }
 mission.markSystem(this.explorerClub_record[galaxyNumber][this.explorerClub_counter]);
 this.explorerClub_counter++;
}
// function to create strings of systems visited and unvisited for display on mission screen
this.explorerClub_systemList = function ()
{
 var visitedName = new Array();
 var unVisitedName = new Array();
 // uses for loop to create array of visited system names.
 var currentGalaxy = galaxyNumber;
 var counter;
 var arrayLength = this.explorerClub_record[galaxyNumber].length;
 for (counter = 0; counter < arrayLength; counter++)
 {
	   visitedName.push(" " + System.systemNameForID(this.explorerClub_record[currentGalaxy][counter]));
 }
 // uses for loop to create array of unvisited system names.
 var systemCounter;
 for (systemCounter = 0; systemCounter < 256;systemCounter++)
 {
	if(this.explorerClub_record[currentGalaxy].indexOf(systemCounter)===-1)
	{
	   unVisitedName.push(" " + System.systemNameForID(systemCounter));
	}
 }
 // puts unvisited in alphabetical order.
 unVisitedName = unVisitedName.sort();
 // slices longest array into maximum of 128 systems and calls mission screen.
 if (visitedName.length <= 128)
 {
	this.explorerClub_unVisitedNameFirstPage = unVisitedName.slice(0,128);
	this.explorerClub_unVisitedNameSecondPage = unVisitedName.slice(128);
	this.explorerClub_visitedName = visitedName;
	this.explorerClub_showScreens = 1;
	this.missionScreenOpportunity();
 }
 if (visitedName.length > 128)
 {
    this.explorerClub_visitedNameFirstPage = visitedName.slice(0,128);
	this.explorerClub_visitedNameSecondPage = visitedName.slice(128);
	this.explorerClub_unVisitedName = unVisitedName;
	this.explorerClub_showScreens = 4;
	this.missionScreenOpportunity();
 }
 }
 // function that defines mile-stones and tests to see if the requisite systems are recorded as visited.
this.explorerClub_mileStones = function ()
{
  // uses the known position of systemID 255 in each galaxy to check if player is using non standard galaxy_seed - if so standard mile-stones are not used.
  var testCoords=[[4.8,40.6,0],[9.6,30.4,0],[31.2,42.4,0],[62.4,34.2,0],[51.2,28.8,0],[86.4,27.6,0],[70.8,4.6,0],[90.8,39.8,0]];
  if (System.infoForSystem(galaxyNumber, 255).coordinates.distanceTo(testCoords[galaxyNumber]) > 0.00001)
  {this.explorerClub_alternateOoniverse = true;}
  if (this.explorerClub_alternateOoniverse) // skip rest of mile-stones check if alternative Ooniverse.
  {
	this.explorerClub_showScreens = 7;
	return;
  }
  // code block to check if the visit record for G1 includes all of the 'Old Worlds'
  if (!this.explorerClub_oldWorlds)
  {
	var oldWorlds = [7,39,46,55,129,147,255];
	this.explorerClub_oldWorlds = this.explorerClub_isArray1_subsetOfArray2(oldWorlds,this.explorerClub_record[0]);
  }
  // code block to check if the visit record for G1 includes the 'Famous Planets' - (from OXP of same name)
  if (!this.explorerClub_famousPlanetsGalOne)
  {
	var famousPlanetsGalOne = [3,7,13,16,18,21,23,28,29,35,36,39,42,44,50,55,62,66,73,86,89,90,93,99,100,101,111,124,126,129,131,132,141,147,150,153,154,172,177,186,188,198,200,221,222,227,228,241,246,250];
	this.explorerClub_famousPlanetsGalOne = this.explorerClub_isArray1_subsetOfArray2(famousPlanetsGalOne,this.explorerClub_record[0]);
  }
  // code block to check if the visit record for G2 includes the 'Famous Planets' - (from OXP of same name)
  if (!this.explorerClub_famousPlanetsGalTwo)
  {
	var famousPlanetsGalTwo = [6,23,24,29,33,42,45,48,53,54,57,58,65,66,74,78,82,88,94,109,113,114,115,122,127,136,140,144,178,188,189,193,202,204,207,221,236];
	this.explorerClub_famousPlanetsGalTwo = this.explorerClub_isArray1_subsetOfArray2(famousPlanetsGalTwo,this.explorerClub_record[1]);
  }
  // code block to check if the visit record for G3 includes the 'Prodigal Suns'
  if (!this.explorerClub_prodigalSuns)
  {
	var prodigalSuns = [133,206];
	this.explorerClub_prodigalSuns = this.explorerClub_isArray1_subsetOfArray2(prodigalSuns,this.explorerClub_record[2]);
  }
  // code block to check if the visit record for G6 inludes the 'Lost Worlds'
  if (!this.explorerClub_lostWorlds)
  {
	var lostWorlds = [33,41,55,202];
	this.explorerClub_lostWorlds = this.explorerClub_isArray1_subsetOfArray2(lostWorlds,this.explorerClub_record[5]);
  }
  // code block to check if the visit record for G7 includes the systems beyond the 'Great Rift'
  if (!this.explorerClub_greatRift)
  {
	var greatRift = [15,17,27,30,61,79,97,110,115,125,130,134,137,139,153,161,173,178,186,191,205,214,218,228,229,237,248];
    this.explorerClub_greatRift = this.explorerClub_isArray1_subsetOfArray2(greatRift,this.explorerClub_record[6]);
  }
  // code block to check if the visit record for G8 includes Oresrati.
  if (!this.explorerClub_oresratiChallenge)
  {
	if (this.explorerClub_record[7].indexOf(162) !== -1)
	{this.explorerClub_oresratiChallenge = true;}
  }
  // code block to check if at least one system has been visited in each Galaxy.
  if (!this.explorerClub_everyGalaxy)
  {if (this.explorerClub_record[0].length !==0 && this.explorerClub_record[1].length !==0 && this.explorerClub_record[2].length !==0 && this.explorerClub_record[3].length !==0 && this.explorerClub_record[4].length !==0 && this.explorerClub_record[5].length !==0 && this.explorerClub_record[6].length !==0 && this.explorerClub_record[7].length !==0)
  {this.explorerClub_everyGalaxy = true;}}
  this.explorerClub_showScreens = 8;
}
 // event handler for mission screens, but in this script called manually from within the script initially. Stops potential mission screen conflicts with any other OXP.
 this.missionScreenOpportunity = function()
 {
  if (!this.explorerClub_showScreens)
  {return;}
  switch (this.explorerClub_showScreens)
	{
		case 1:
		{
			mission.runScreen({title: "G" + (galaxyNumber+1)+": Systems visited", message: this.explorerClub_visitedName, background : "Explorers_Club.png"});
			this.explorerClub_showScreens = 2;
		}break;
		case 2:
		{
			mission.runScreen({title: "G" + (galaxyNumber+1)+": Systems to visit (page 1)", message: this.explorerClub_unVisitedNameFirstPage, background : "Explorers_Club.png"});
			this.explorerClub_showScreens = 3;
		}break;
		case 3:
		{
			mission.runScreen({title: "G" + (galaxyNumber+1)+": Systems to visit (page 2)", message: this.explorerClub_unVisitedNameSecondPage, background : "Explorers_Club.png"});
			delete this.explorerClub_showScreens;
			this.explorerClub_mileStones();
		}break;
		case 4:
		{
			mission.runScreen({title: "G" + (galaxyNumber+1)+": Systems visited(page 1)", message: this.explorerClub_visitedNameFirstPage, background : "Explorers_Club.png"});
			this.explorerClub_showScreens = 5;
		}break;
		case 5:
		{
			mission.runScreen({title: "G" + (galaxyNumber+1)+": Systems visited(page 2)", message: this.explorerClub_visitedNameSecondPage, background : "Explorers_Club.png"});
			this.explorerClub_showScreens = 6;
		}break;
		case 6:
		{
			mission.runScreen({title: "G" + (galaxyNumber+1)+": Systems to visit", message: this.explorerClub_unVisitedName, background : "Explorers_Club.png"});
			delete this.explorerClub_showScreens;
			this.explorerClub_mileStones();
		}break;
		case 7:
		{
			mission.runScreen({title:"Explorers' Club Mile-stones", messageKey:"explorer_club_alternate_ooniverse", background : "Explorers_Club.png"});
			delete this.explorerClub_showScreens;
		}break;
		case 8:
		{
			mission.runScreen({title:"Explorers' Club Mile-stones", messageKey:"explorer_club_milestones", background : "Explorers_Club.png"});
			delete this.explorerClub_showScreens;
			if (this.explorerClub_oldWorlds)
			{mission.addMessageText("\n**You have visited all the 'Old Worlds' in Galaxy 1.**\n");}
			if (this.explorerClub_famousPlanetsGalOne)
			{mission.addMessageText("**You have visited all the 'Famous Planets' in Galaxy 1.**\n");}
			if (this.explorerClub_famousPlanetsGalTwo)
			{mission.addMessageText("**You have visited all the 'Famous Planets' in Galaxy 2.**\n");}
			if (this.explorerClub_prodigalSuns)
			{mission.addMessageText("**You have visited both systems in the 'Prodigal Suns' sector in Galaxy 3.**\n");}
		    if (this.explorerClub_lostWorlds)
			{mission.addMessageText("**You have visited the 'Lost Worlds' systems in Galaxy 6.**\n");}
			if (this.explorerClub_greatRift)
			{mission.addMessageText("**You have visited all the systems across the 'Great Rift' in Galaxy 7.**\n")}
			if (this.explorerClub_oresratiChallenge)
			{mission.addMessageText("**You have visited Oresrati in Galaxy 8.**\n");}
			if (this.explorerClub_everyGalaxy)
			{mission.addMessageText("**You have been around the 'eight' at least once.**\n");}
			if (!this.explorerClub_oldWorlds)
			{mission.addMessageText("You have not visited all the 'Old Worlds' in Galaxy 1.\n");}
			if (!this.explorerClub_famousPlanetsGalOne)
			{mission.addMessageText("You have not visited all the 'Famous Planets' in Galaxy 1.\n");}
			if (!this.explorerClub_famousPlanetsGalTwo)
			{mission.addMessageText("You have not visited all the 'Famous Planets' in Galaxy 2.\n");}
			if (!this.explorerClub_prodigalSuns)
			{mission.addMessageText("You have not visited both systems in the 'Prodigal Suns' sector in Galaxy 3.\n");}
		    if (!this.explorerClub_lostWorlds)
			{mission.addMessageText("You have not visited the 'Lost Worlds' systems in Galaxy 6.\n");}
			if (!this.explorerClub_greatRift)
			{mission.addMessageText("You have not visited all the systems across the 'Great Rift' in Galaxy 7.\n");}
			if (!this.explorerClub_oresratiChallenge)
			{mission.addMessageText("You have not visited Oresrati in Galaxy 8.\n");}
			if (!this.explorerClub_everyGalaxy)
			{mission.addMessageText("You have not yet done your first 'eight'.\n");}
		}break;
	}
}
// function to check if array1 is subset of array2 - order does not matter. Used to check mile-stones if mile-stone involves checking for multiple systems in this.explorerClub_record.
this.explorerClub_isArray1_subsetOfArray2 = function (array1,array2)
{
 if ((!array1 || !array2) || (array1.length > array2.length))
 {return false;}
 if (array1.length === array2.filter(function(value){return this.indexOf(value) >= 0},array1).length)
 {return true;}
 else {return false;}
}

// the following functions and variables are all used by the experimental snapshot feature.

this.explorerClub_snapshotDelay = 0.25;// the delay for the timer that calls for a snapshot to be taken following an appropriate shipExitedWitchspace or shipLaunchedFromStation event. If you experience JS timeout errors for scripts other than explorer_club_snapshot.js at the point the snapshot is taken try increasing this value in increments of 0.25, until errors stop. Value is in seconds.
this.explorerClub_releasePlayer = 1.25;// the delay for the failsafe that releases the player after a snapshot has been taken and the player has not already been released. Necessary in case of explorer_club_snapshot.js time-out and failure to return this.explorerClub_snapshotTaken as true. Should always be at least 1 second greater than this.explorerClub_snapshotDelay.

// the default values work well on a 7 year old Windows XP laptop which is reasonably well maintained (hard disk about 70% full, regular defrag and disk maintenance (windows chkdsk). The same machine prior to a windows chkdsk was producing frequent timeouts presumably due to increased hard disk write-times.

this.explorerClub_arrivalSnapshotCheck = function()
{
    if (!this.explorerClub_snapshotOnArrival)//will only be true if snapshot feature is on, and a visit has been recorded on arrival from witchspace
	{return;}
	delete this.explorerClub_snapshotOnArrival;
	this.explorerClub_originalHUD = player.ship.hud;// record current hud
	player.ship.hud = "exclub-photo-hud.plist"; // change to explorers' club HUD (thanks Wildeblood for the HUD).
	this.explorerClub_callback = addFrameCallback(this.explorerClub_freezePlayer.bind(this));
}
// called via Frame callback via shipExitedWitchspace. On first frame, records player position and orientation and sets up one shot timers to a) take snapshot, b) remove Frame callback if explorer_club_snapshot.js has not returned true after a specified delay (the difference between this.explorerClub_snapshotDelay and this.explorerClub_releasePlayer). Until removed the player will be locked into position. Timers are used to minimise impact of takeSnapShot JS overhead on other scripts. takeSnapShot is isolated in seperate worldScript to prevent potential JS timeout crashing main worldScript.
this.explorerClub_freezePlayer = function()
{   if (!this.explorerClub_originalPosition)
    {
		this.explorerClub_originalPosition = player.ship.position;
		this.explorerClub_originalOrientation = player.ship.orientation;
		player.consoleMessage("Explorers' Club record of "+System.systemNameForID(system.ID)+".",this.explorerClub_releasePlayer);
		this.releaseFailSafe = clock.absoluteSeconds + this.explorerClub_releasePlayer;
		this.snapshotTimer = new Timer(this,worldScripts["explorer_club_snapshot.js"].explorerClub_snapshot,this.explorerClub_snapshotDelay);
	}
	player.ship.position = this.explorerClub_originalPosition;
	player.ship.orientation = this.explorerClub_originalOrientation;
	if (this.explorerClub_snapshotTaken || clock.absoluteSeconds > this.releaseFailSafe)
	{
		this.explorerClub_removeCallback();
		if (this.explorerClub_snapshotTaken){delete this.explorerClub_snapshotTaken; log(this.name,"Snapshot on arrival successful taken - call back removed normally.");}
		else {log(this.name,"Check that snapshot on arrival taken - callback removed by fail safe.");}
	}
}
	
this.shipLaunchedFromStation = function(station)
{
	if (!this.explorerClub_snapshotOnLaunch)//will only be true if snapshot feature is on, and a visit has been recorded on prior docking with main station
	{return;}
	delete this.explorerClub_snapshotOnLaunch;
	this.explorerClub_originalHUD = player.ship.hud;// record current hud
	player.ship.hud = "exclub-photo-hud.plist"; // change to explorers' club HUD (thanks Wildeblood for the HUD).
	this.explorerClub_callback = addFrameCallback(this.explorerClub_positionPlayer.bind(this));
}
// called via Frame callback via shipLaunchedFromStation. On first frame, records original player position and orientation, before moving player 250km backwards from station/planet.  On second frame sets up one shot timers to a) take snapshot, b) remove Frame callback and restore player to original position and orientation if explorer_club_snapshot.js has not returned true after a specified delay (the difference between this.explorerClub_snapshotDelay and this.explorerClub_releasePlayer). Until removed the player will then be locked into position. Timers are used to minimise impact of takeSnapShot JS overhead on other scripts. takeSnapShot is isolated in seperate worldScript to prevent potential JS timeout crashing main worldScript.
this.explorerClub_positionPlayer = function()
{
 if (!this.explorerClub_playerMoved)
 {
	this.explorerClub_playerMoved = true;
	this.explorerClub_originalPosition = player.ship.position;
	this.explorerClub_originalOrientation = player.ship.orientation;
	player.ship.position = this.explorerClub_originalPosition.subtract(player.ship.vectorForward.multiply(250000));
	this.explorerClub_newPosition = player.ship.position;
 }
 else if (this.explorerClub_playerMoved && !this.explorerClub_snapshotUnderway)
 {
	this.explorerClub_snapshotUnderway = true;
	player.consoleMessage("Explorers' Club record of "+System.systemNameForID(system.ID)+".",this.explorerClub_releasePlayer);
	this.snapshotTimer = new Timer(this,worldScripts["explorer_club_snapshot.js"].explorerClub_snapshot,this.explorerClub_snapshotDelay);
	this.releaseFailSafe = clock.absoluteSeconds + this.explorerClub_releasePlayer;
 }
 else if (this.explorerClub_playerMoved && this.explorerClub_snapshotUnderway)
 {
 	player.ship.position = this.explorerClub_newPosition;
	player.ship.orientation = this.explorerClub_originalOrientation;
	if (this.explorerClub_snapshotTaken || clock.absoluteSeconds > this.releaseFailSafe)
	{
		this.explorerClub_removeCallback();
		if (this.explorerClub_snapshotTaken){delete this.explorerClub_snapshotTaken; log(this.name,"Snapshot on launch successful taken - call back removed normally.");}
		else {log(this.name,"Check that snapshot on launch taken - callback removed by fail safe.");}
	}
 }
}
this.explorerClub_removeCallback = function()
{
    removeFrameCallback(this.explorerClub_callback);
	delete this.explorerClub_callback;
	if (this.explorerClub_newPosition) // only true when snapshot taken from station launch
    {player.ship.position = this.explorerClub_originalPosition.add(player.ship.vectorForward.multiply(500));} // player is returned to a position 500m ahead from where started on launch, to prevent collision with any ship that has launched subsequently.
	player.ship.orientation = this.explorerClub_originalOrientation;
	player.ship.hud = this.explorerClub_originalHUD;
	delete this.explorerClub_originalHUD; 
	delete this.explorerClub_playerMoved;
	delete this.explorerClub_snapshotUnderway;
	delete this.explorerClub_originalPosition;
	delete this.explorerClub_originalOrientation;
	delete this.explorerClub_newPosition;
	delete this.releaseFaileSafe;
}
